# -*- coding: utf-8 -*-

"""
Gestion du menu d’administration du portail.
"""

from collections import defaultdict
from django.apps import apps
from django.core.cache import cache
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse
from django.utils.encoding import smart_str
import unicodedata

from .models import Permission

ADMIN_MENU_CACHE_KEY = "admin_menu_for_user_{}_ecole_{}"


class AdminMenu(object):
    """
    Singleton de gestion du menu d’administration
    """

    _instance = None

    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = cls()

        return cls._instance

    def __init__(self):
        assert not self._instance

        self._menu_items = []

        permissions = defaultdict(set)

        for permission, ecole in Permission.objects.all().values_list("ident", "ecole"):
            if ecole is None:
                # Permission globale, commune à toutes les écoles
                assert permission not in permissions
                permissions[permission] = None
            else:
                permissions[permission].add(ecole)

        permissions["root"] = None

        for app in apps.get_app_configs():
            admin_menu = getattr(app, "admin_menu", None)
            if admin_menu is None:
                continue

            for permission, menu_title, view in admin_menu:
                ecoles = permissions.get(permission, -1)  # None is a valid value
                if ecoles == -1:
                    raise ImproperlyConfigured(f"Unknown permission ‘{permission}’ for app ‘{app.verbose_name}’")

                url = reverse(view)

                self._menu_items.append((permission, ecoles, smart_str(menu_title), url))

        def sort_key(item):
            item = unicodedata.normalize("NFD", item[2]).encode("ascii", "ignore")
            return item.lower()

        self._menu_items.sort(key=sort_key)

    def get_menu_for(self, request):
        if request.user.is_anonymous:
            return []

        if request.user.is_superuser:
            cache_name = f"admin_menu_for_superuser_ecole_{request.ecole.id}"
        else:
            cache_name = ADMIN_MENU_CACHE_KEY.format(request.user.id, request.ecole.id)

        menu = cache.get(cache_name)

        if menu is None:
            menu = self._build_menu_for(request)
            cache.set(cache_name, menu, 3600)

        return menu

    def _build_menu_for(self, request):
        return list(self._menu_entries_for(request))

    def _menu_entries_for(self, request):
        user = request.user
        ecole_id = request.ecole.id

        for permission, ecoles, title, url in self._menu_items:
            if ecoles is not None and ecole_id not in ecoles:
                # Cette permission, donc cet élément de menu, ne s’applique pas à cette
                # école
                continue

            if user.check_permission(request, permission):
                yield {"url": url, "title": title}
